@@ -34,6 +34,7 @@ import zipfile
 from common import gajim
 from plugins import GajimPlugin
 from plugins.helpers import log_calls, log
+from conversation_textview import ConversationTextview
 from dialogs import WarningDialog, HigDialog, YesNoDialog
 from plugins.gui import GajimPluginConfigDialog
 
@@ -63,7 +64,8 @@ class PluginInstaller(GajimPlugin):
         self.description = _('Install and upgrade plugins from ftp')
         self.config_dialog = PluginInstallerPluginConfigDialog(self)
         self.config_default_values = {'ftp_server': ('ftp.gajim.org', ''),
-                                      'check_update': (True, ''),}
+                                      'check_update': (True, ''),
+                                      'TLS': (True, ''),}
         self.window = None
         self.progressbar = None
         self.available_plugins_model = None
@@ -73,6 +75,12 @@ class PluginInstaller(GajimPlugin):
         icon = gtk.Image()
         self.def_icon = icon.render_icon(gtk.STOCK_PREFERENCES,
             gtk.ICON_SIZE_MENU)
+        if gajim.version.startswith('0.15'):
+            self.server_folder = 'plugins_0.15'
+        elif gajim.version.startswith('0.16.10'):
+            self.server_folder = 'plugins_gtk3'
+        else:
+            self.server_folder = 'plugins_0.16'
 
     @log_calls('PluginInstallerPlugin')
     def activate(self):
@@ -98,7 +106,7 @@ class PluginInstaller(GajimPlugin):
                 '\n%s') % plugins_str, on_response_yes=open_update)
 
     def ftp_connect(self):
-        if sys.version_info[:2] > (2, 6):
+        if sys.version_info[:2] > (2, 6) and self.config['TLS'] :
             con = ftplib.FTP_TLS(self.config['ftp_server'])
             con.login()
             con.prot_p()
@@ -113,20 +121,28 @@ class PluginInstaller(GajimPlugin):
             try:
                 to_update = []
                 con = self.ftp_connect()
-                con.cwd('plugins')
+                con.cwd(self.server_folder)
                 con.retrbinary('RETR manifests.zip', ftp.handleDownload)
                 zip_file = zipfile.ZipFile(ftp.buffer_)
                 manifest_list = zip_file.namelist()
                 for filename in manifest_list:
-                    ftp.config.readfp(zip_file.open(filename))
-                    local_version = ftp.get_plugin_version(ftp.config.get(
+                    config = ConfigParser.ConfigParser()
+                    config.readfp(zip_file.open(filename))
+                    if not config.has_section('info'):
+                        continue
+                    opts = config.options('info')
+                    if 'name' not in opts or 'version' not in opts or \
+                    'description' not in opts or 'authors' not in opts or \
+                    'homepage' not in opts:
+                        continue
+                    local_version = ftp.get_plugin_version(config.get(
                         'info', 'name'))
                     if local_version:
                         local = convert_version_to_list(local_version)
-                        remote = convert_version_to_list(ftp.config.get('info',
+                        remote = convert_version_to_list(config.get('info',
                             'version'))
                         if remote > local:
-                            to_update.append(ftp.config.get('info', 'name'))
+                            to_update.append(config.get('info', 'name'))
                 con.quit()
                 gobject.idle_add(self.warn_update, to_update)
             except Exception, e:
@@ -140,7 +156,7 @@ class PluginInstaller(GajimPlugin):
     def deactivate(self):
         self.pl_menuitem.disconnect(self.id_)
         if hasattr(self, 'page_num'):
-            self.notebook.remove_page(self.page_num)
+            self.notebook.remove_page(self.notebook.page_num(self.hpaned))
             self.notebook.set_current_page(0)
             for id_, widget in self.connected_ids.items():
                 widget.disconnect(id_)
@@ -168,15 +184,16 @@ class PluginInstaller(GajimPlugin):
         self.GTK_BUILDER_FILE_PATH = self.local_file_path('config_dialog.ui')
         self.xml = gtk.Builder()
         self.xml.set_translation_domain('gajim_plugins')
-        self.xml.add_objects_from_file(self.GTK_BUILDER_FILE_PATH, ['hpaned2'])
-        hpaned = self.xml.get_object('hpaned2')
-        self.page_num = self.notebook.append_page(hpaned,
+        self.xml.add_objects_from_file(self.GTK_BUILDER_FILE_PATH, ['hpaned2',
+            'image1'])
+        self.hpaned = self.xml.get_object('hpaned2')
+        self.page_num = self.notebook.append_page(self.hpaned,
             gtk.Label(_('Available')))
 
         widgets_to_extract = ('plugin_name_label1',
         'available_treeview', 'progressbar', 'inslall_upgrade_button',
         'plugin_authors_label1', 'plugin_authors_label1',
-        'plugin_homepage_linkbutton1', 'plugin_description_textview1')
+        'plugin_homepage_linkbutton1')
 
         for widget_name in widgets_to_extract:
             setattr(self, widget_name, self.xml.get_object(widget_name))
@@ -239,6 +256,11 @@ class PluginInstaller(GajimPlugin):
         selection.set_mode(gtk.SELECTION_SINGLE)
 
         self._clear_available_plugin_info()
+
+        self.plugin_description_textview = ConversationTextview(None)
+        sw = self.xml.get_object('scrolledwindow1')
+        sw.add(self.plugin_description_textview.tv)
+
         self.xml.connect_signals(self)
         self.window.show_all()
 
@@ -261,7 +283,10 @@ class PluginInstaller(GajimPlugin):
             self.inslall_upgrade_button.set_property('sensitive', True)
 
     def on_notebook_switch_page(self, widget, page, page_num):
-        if not hasattr(self, 'ftp') and self.page_num == page_num:
+        tab_label_text = self.notebook.get_tab_label_text(self.hpaned)
+        if tab_label_text != (_('Available')):
+            return
+        if not hasattr(self, 'ftp'):
             self.available_plugins_model.clear()
             self.progressbar.show()
             self.ftp = Ftp(self)
@@ -341,6 +366,11 @@ class PluginInstaller(GajimPlugin):
 
     def available_plugins_treeview_selection_changed(self, treeview_selection):
         model, iter = treeview_selection.get_selected()
+        self.xml.get_object('scrolledwindow1').get_children()[0].destroy()
+        self.plugin_description_textview = ConversationTextview(None)
+        sw = self.xml.get_object('scrolledwindow1')
+        sw.add(self.plugin_description_textview.tv)
+        sw.show_all()
         if iter:
             self.plugin_name_label1.set_text(model.get_value(iter, C_NAME))
             self.plugin_authors_label1.set_text(model.get_value(iter, C_AUTHORS))
@@ -351,9 +381,24 @@ class PluginInstaller(GajimPlugin):
             label = self.plugin_homepage_linkbutton1.get_children()[0]
             label.set_ellipsize(pango.ELLIPSIZE_END)
             self.plugin_homepage_linkbutton1.set_property('sensitive', True)
-            desc_textbuffer = self.plugin_description_textview1.get_buffer()
-            desc_textbuffer.set_text(_(model.get_value(iter, C_DESCRIPTION)))
-            self.plugin_description_textview1.set_property('sensitive', True)
+            desc = _(model.get_value(iter, C_DESCRIPTION))
+            if not desc.startswith('<body '):
+                desc = '<body  xmlns=\'http://www.w3.org/1999/xhtml\'>' + \
+                    desc + ' </body>'
+                desc = desc.replace('\n', '<br/>')
+            import inspect
+            args = inspect.getargspec(
+                self.plugin_description_textview.tv.display_html)
+            gajim_version = list(gajim.version.split('-', 1)[0].split('.'))
+            if (gajim_version >= [0, 16]):
+                # Gajim_0.16
+                self.plugin_description_textview.tv.display_html(desc,
+                    self.plugin_description_textview.tv,
+                    self.plugin_description_textview)
+            else:
+                self.plugin_description_textview.tv.display_html(desc,
+                    self.plugin_description_textview)
+            self.plugin_description_textview.tv.set_property('sensitive', True)
         else:
             self._clear_available_plugin_info()
 
@@ -364,10 +409,6 @@ class PluginInstaller(GajimPlugin):
         self.plugin_homepage_linkbutton1.set_label('')
         self.plugin_homepage_linkbutton1.set_property('sensitive', False)
 
-        desc_textbuffer = self.plugin_description_textview1.get_buffer()
-        desc_textbuffer.set_text('')
-        self.plugin_description_textview1.set_property('sensitive', False)
-
     def scan_dir_for_plugin(self, path):
         plugins_found = []
         conf = ConfigParser.ConfigParser()
@@ -440,7 +481,9 @@ class PluginInstaller(GajimPlugin):
                 root_iter = self.available_plugins_model.get_iter_root()
                 selection.select_iter(root_iter)
         scr_win = self.xml.get_object('scrolledwindow2')
-        scr_win.get_vadjustment().set_value(0)
+        vadjustment = scr_win.get_vadjustment()
+        if vadjustment:
+            vadjustment.set_value(0)
 
 
 class Ftp(threading.Thread):
@@ -450,7 +493,6 @@ class Ftp(threading.Thread):
         self.window = plugin.window
         self.progressbar = plugin.progressbar
         self.model = plugin.available_plugins_model
-        self.config = ConfigParser.ConfigParser()
         self.buffer_ = io.BytesIO()
         self.remote_dirs = None
         self.append_to_model = True
@@ -477,11 +519,12 @@ class Ftp(threading.Thread):
             gobject.idle_add(self.progressbar.set_text,
                 _('Connecting to server'))
             self.ftp = self.plugin.ftp_connect()
-            self.ftp.cwd('plugins')
+            self.ftp.cwd(self.plugin.server_folder)
             if not self.remote_dirs:
                 gobject.idle_add(self.progressbar.set_text,
                     _('Scan files on the server'))
-                self.ftp.retrbinary('RETR manifests_images.zip', self.handleDownload)
+                self.ftp.retrbinary('RETR manifests_images.zip',
+                    self.handleDownload)
                 zip_file = zipfile.ZipFile(self.buffer_)
                 manifest_list = zip_file.namelist()
                 progress_step = 1.0 / len(manifest_list)
@@ -494,13 +537,22 @@ class Ftp(threading.Thread):
                     gobject.idle_add(self.progressbar.set_text,
                         _('Reading "%s"') % dir_)
 
-                    self.config.readfp(zip_file.open(filename))
+                    config = ConfigParser.ConfigParser()
+                    config.readfp(zip_file.open(filename))
+                    if not config.has_section('info'):
+                        continue
+                    opts = config.options('info')
+                    if 'name' not in opts or 'version' not in opts or \
+                    'description' not in opts or 'authors' not in opts or \
+                    'homepage' not in opts:
+                        continue
+
                     local_version = self.get_plugin_version(
-                        self.config.get('info', 'name'))
+                        config.get('info', 'name'))
                     upgrade = False
                     if self.upgrading and local_version:
                         local = convert_version_to_list(local_version)
-                        remote = convert_version_to_list(self.config.get('info',
+                        remote = convert_version_to_list(config.get('info',
                             'version'))
                         if remote > local:
                             upgrade = True
@@ -522,11 +574,11 @@ class Ftp(threading.Thread):
                         local_dir = os.path.join(user_dir, dir_)
 
                     gobject.idle_add(self.model_append, [def_icon, dir_,
-                        self.config.get('info', 'name'), local_version,
-                        self.config.get('info', 'version'), upgrade,
-                        self.config.get('info', 'description'),
-                        self.config.get('info', 'authors'),
-                        self.config.get('info', 'homepage'), ])
+                        config.get('info', 'name'), local_version,
+                        config.get('info', 'version'), upgrade,
+                        config.get('info', 'description'),
+                        config.get('info', 'authors'),
+                        config.get('info', 'homepage'), ])
                 self.ftp.quit()
             gobject.idle_add(self.progressbar.set_fraction, 0)
             if self.remote_dirs:
@@ -565,7 +617,7 @@ class Ftp(threading.Thread):
                     else:
                         files.append(i[1:])
             dirs, files = [], []
-            nlstr('/plugins/' + remote_dir)
+            nlstr('/%s/%s' % (self.plugin.server_folder, remote_dir))
 
             base_dir, user_dir = gajim.PLUGINS_DIRS
             if not os.path.isdir(user_dir):
@@ -577,18 +629,17 @@ class Ftp(threading.Thread):
 
             # creating dirs
             for dir_ in dirs:
-                try:
-                    os.mkdir(os.path.join(local_dir, dir_))
-                except OSError, e:
-                    if str(e).startswith('[Errno 17]'):
-                        continue
-                    raise
+                dir_ = dir_.replace(self.plugin.server_folder, 'plugins')
+                if os.path.exists(os.path.join(local_dir, dir_)):
+                    continue
+                os.mkdir(os.path.join(local_dir, dir_))
 
             # downloading files
             for filename in files:
                 gobject.idle_add(self.progressbar.set_text,
                     _('Downloading "%s"') % filename)
-                full_filename = os.path.join(local_dir, filename)
+                full_filename = os.path.join(local_dir, filename.replace(
+                    self.plugin.server_folder, 'plugins'))
                 try:
                     self.ftp.retrbinary('RETR /%s' % filename,
                         open(full_filename, 'wb').write)
@@ -620,6 +671,7 @@ class PluginInstallerPluginConfigDialog(GajimPluginConfigDialog):
         widget.set_text(str(self.plugin.config['ftp_server']))
         self.xml.get_object('check_update').set_active(
             self.plugin.config['check_update'])
+        self.xml.get_object('TLS').set_active(self.plugin.config['TLS'])
 
     def on_hide(self, widget):
         widget = self.xml.get_object('ftp_server')
@@ -627,3 +679,6 @@ class PluginInstallerPluginConfigDialog(GajimPluginConfigDialog):
 
     def on_check_update_toggled(self, widget):
         self.plugin.config['check_update'] = widget.get_active()
+
+    def on_tls_toggled(self, widget):
+        self.plugin.config['TLS'] = widget.get_active()
