@@ -1,4 +1,4 @@
-# Copyright (C) 2009  red-agent <hell.director@gmail.com>
+# Copyright (C) 2009-2010  Alexander Cherniuk <ts33kr@gmail.com>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,18 +15,24 @@
 
 class BaseError(Exception):
     """
-    Common base for errors which relate to a specific command. Encapsulates
-    everything needed to identify a command, by either its object or name.
+    Common base for errors which relate to a specific command.
+    Encapsulates everything needed to identify a command, by either its
+    object or name.
     """
 
     def __init__(self, message, command=None, name=None):
+        self.message = message
+
         self.command = command
         self.name = name
 
         if command and not name:
             self.name = command.first_name
 
-        super(BaseError, self).__init__(message)
+        super(BaseError, self).__init__()
+
+    def __str__(self):
+        return self.message
 
 class DefinitionError(BaseError):
     """
@@ -39,3 +45,9 @@ class CommandError(BaseError):
     Used to indicate errors occured during command execution.
     """
     pass
+
+class NoCommandError(BaseError):
+    """
+    Used to indicate an inability to find the specified command.
+    """
+    pass
