@@ -1,7 +1,7 @@
 # -*- coding:utf-8 -*-
 ## src/common/logger.py
 ##
-## Copyright (C) 2003-2012 Yann Leboulanger <asterix AT lagaule.org>
+## Copyright (C) 2003-2014 Yann Leboulanger <asterix AT lagaule.org>
 ## Copyright (C) 2004-2005 Vincent Hanquez <tab AT snarc.org>
 ## Copyright (C) 2005-2006 Nikos Kouremenos <kourem AT gmail.com>
 ## Copyright (C) 2006 Dimitur Kirov <dkirov AT gmail.com>
@@ -34,6 +34,7 @@ import time
 import datetime
 from gzip import GzipFile
 from cStringIO import StringIO
+import gobject
 
 import exceptions
 import gajim
@@ -91,7 +92,8 @@ class Constants:
                 self.TYPE_RSS,
                 self.TYPE_WEATHER,
                 self.TYPE_MRIM,
-        ) = range(14)
+                self.TYPE_NO_TRANSPORT,
+        ) = range(15)
 
         (
                 self.SUBSCRIPTION_NONE,
@@ -106,6 +108,7 @@ class Logger:
     def __init__(self):
         self.jids_already_in = [] # holds jids that we already have in DB
         self.con = None
+        self.commit_timout_id = None
 
         if not os.path.exists(LOG_DB_PATH):
             # this can happen only the first time (the time we create the db)
@@ -146,7 +149,8 @@ class Logger:
 
     def attach_cache_database(self):
         try:
-            self.cur.execute("ATTACH DATABASE '%s' AS cache" % CACHE_DB_PATH)
+            self.cur.execute("ATTACH DATABASE '%s' AS cache" % \
+                CACHE_DB_PATH.replace("'", "''"))
         except sqlite.Error, e:
             log.debug("Failed to attach cache database: %s" % str(e))
 
@@ -163,15 +167,25 @@ class Logger:
         self.open_db()
         self.get_jids_already_in_db()
 
+    def _really_commit(self):
+        try:
+            self.con.commit()
+        except sqlite.OperationalError, e:
+            print >> sys.stderr, str(e)
+        self.commit_timout_id = None
+        return False
+
+    def _timeout_commit(self):
+        if self.commit_timout_id:
+            return
+        self.commit_timout_id = gobject.timeout_add(500, self._really_commit)
+
     def simple_commit(self, sql_to_commit):
         """
         Helper to commit
         """
         self.cur.execute(sql_to_commit)
-        try:
-            self.con.commit()
-        except sqlite.OperationalError, e:
-            print >> sys.stderr, str(e)
+        self._timeout_commit()
 
     def get_jids_already_in_db(self):
         try:
@@ -327,6 +341,8 @@ class Logger:
             return constants.TYPE_WEATHER
         if type_ == 'mrim':
             return constants.TYPE_MRIM
+        if type_ == 'jabber':
+            return constants.TYPE_NO_TRANSPORT
         return None
 
     def convert_api_values_to_human_transport_type(self, type_id):
@@ -361,6 +377,8 @@ class Logger:
             return 'weather'
         if type_id == constants.TYPE_MRIM:
             return 'mrim'
+        if type_id == constants.TYPE_NO_TRANSPORT:
+            return 'jabber'
 
     def convert_human_subscription_values_to_db_api_values(self, sub):
         """
@@ -398,12 +416,14 @@ class Logger:
         except sqlite.OperationalError, e:
             raise exceptions.PysqliteOperationalError(str(e))
         message_id = None
-        try:
-            self.con.commit()
-            if write_unread:
+        if write_unread:
+            try:
+                self.con.commit()
                 message_id = self.cur.lastrowid
-        except sqlite.OperationalError, e:
-            print >> sys.stderr, str(e)
+            except sqlite.OperationalError, e:
+                print >> sys.stderr, str(e)
+        else:
+            self._timeout_commit()
         if message_id:
             self.insert_unread_events(message_id, values[0])
         return message_id
@@ -629,7 +649,8 @@ class Logger:
         results = self.cur.fetchall()
         return results
 
-    def get_search_results_for_query(self, jid, query, account):
+    def get_search_results_for_query(self, jid, query, account, year=False,
+        month=False, day=False):
         """
         Returns contact_name, time, kind, show, message
 
@@ -652,7 +673,19 @@ class Logger:
         else: # user just typed something, we search in message column
             where_sql, jid_tuple = self._build_contact_where(account, jid)
             like_sql = '%' + query.replace("'", "''") + '%'
-            self.cur.execute('''
+            if year:
+                start_of_day = self.get_unix_time_from_date(year, month, day)
+                seconds_in_a_day = 86400 # 60 * 60 * 24
+                last_second_of_day = start_of_day + seconds_in_a_day - 1
+                self.cur.execute('''
+                SELECT contact_name, time, kind, show, message, subject FROM logs
+                WHERE (%s) AND message LIKE '%s'
+                AND time BETWEEN %d AND %d
+                ORDER BY time
+                ''' % (where_sql, like_sql, start_of_day, last_second_of_day),
+                    jid_tuple)
+            else:
+                self.cur.execute('''
                 SELECT contact_name, time, kind, show, message, subject FROM logs
                 WHERE (%s) AND message LIKE '%s'
                 ORDER BY time
@@ -909,10 +942,7 @@ class Logger:
                 VALUES (?, ?, ?, ?);
                 ''', (hash_method, hash_, buffer(data), int(time.time())))
         # (1) -- note above
-        try:
-            self.con.commit()
-        except sqlite.OperationalError, e:
-            print >> sys.stderr, str(e)
+        self._timeout_commit()
 
     def update_caps_time(self, method, hash_):
         sql = '''UPDATE caps_cache SET last_seen = %d
@@ -950,9 +980,9 @@ class Logger:
         # Fill roster tables with the new roster
         for jid in roster:
             self.add_or_update_contact(account_jid, jid, roster[jid]['name'],
-                    roster[jid]['subscription'], roster[jid]['ask'],
-                    roster[jid]['groups'], commit=False)
-        self.con.commit()
+                roster[jid]['subscription'], roster[jid]['ask'],
+                roster[jid]['groups'], commit=False)
+        self._timeout_commit()
 
         # At this point, we are sure the replacement works properly so we can
         # set the new roster_version value.
@@ -974,7 +1004,7 @@ class Logger:
         self.cur.execute(
                 'DELETE FROM roster_entry WHERE account_jid_id=? AND jid_id=?',
                 (account_jid_id, jid_id))
-        self.con.commit()
+        self._timeout_commit()
 
     def add_or_update_contact(self, account_jid, jid, name, sub, ask, groups,
     commit=True):
@@ -1009,7 +1039,7 @@ class Logger:
                 self.convert_human_subscription_values_to_db_api_values(sub),
                 bool(ask)))
         if commit:
-            self.con.commit()
+            self._timeout_commit()
 
     def get_roster(self, account_jid):
         """
@@ -1062,7 +1092,7 @@ class Logger:
                 (account_jid_id,))
         self.cur.execute('DELETE FROM roster_group WHERE account_jid_id=?',
                 (account_jid_id,))
-        self.con.commit()
+        self._timeout_commit()
 
     def save_if_not_exists(self, with_, direction, tim, msg='', nick=None):
         if tim:
