@@ -21,7 +21,6 @@ import logging
 log = logging.getLogger('gajim.c.z.zeroconf_avahi')
 
 try:
-    import dbus.glib
     import dbus.exceptions
 except ImportError, e:
     pass
@@ -285,9 +284,10 @@ class Zeroconf:
 
         state = self.server.GetState()
         if state == self.avahi.SERVER_RUNNING:
-            self.create_service()
-            self.announced = True
-            return True
+            if self.create_service():
+            	self.announced = True
+            	return True
+            return False
 
     def remove_announce(self):
         if self.announced == False:
