@@ -1,7 +1,7 @@
 # -*- coding:utf-8 -*-
 ## src/gtkgui_helpers.py
 ##
-## Copyright (C) 2003-2010 Yann Leboulanger <asterix AT lagaule.org>
+## Copyright (C) 2003-2014 Yann Leboulanger <asterix AT lagaule.org>
 ## Copyright (C) 2005-2006 Dimitur Kirov <dkirov AT gmail.com>
 ## Copyright (C) 2005-2007 Nikos Kouremenos <kourem AT gmail.com>
 ## Copyright (C) 2006 Travis Shirk <travis AT pobox.com>
@@ -44,11 +44,12 @@ from common import gajim
 gtk_icon_theme = gtk.icon_theme_get_default()
 gtk_icon_theme.append_search_path(gajim.ICONS_DIR)
 
-def get_icon_pixmap(icon_name, size=16):
+def get_icon_pixmap(icon_name, size=16, quiet=False):
     try:
         return gtk_icon_theme.load_icon(icon_name, size, 0)
     except gobject.GError, e:
-        log.error('Unable to load icon %s: %s' % (icon_name, str(e)))
+        if not quiet:
+            log.error('Unable to load icon %s: %s' % (icon_name, str(e)))
 
 def get_icon_path(icon_name, size=16):
     try:
@@ -173,13 +174,11 @@ def get_default_font():
     Xfce and last KDE it returns None on failure or else a string 'Font Size'
     """
     try:
-        import gconf
-        # in try because daemon may not be there
-        client = gconf.client_get_default()
-
-        return client.get_string('/desktop/gnome/interface/font_name'
-                ).decode('utf-8')
-    except Exception:
+        from gconf import client_get_default
+        client = client_get_default()
+        value = client.get_string("/desktop/gnome/interface/font_name")
+        return value.decode("utf8")
+    except ImportError, glib.GError:
         pass
 
     # try to get Xfce default font
@@ -223,29 +222,6 @@ def get_default_font():
 
     return None
 
-def autodetect_browser_mailer():
-    # recognize the environment and set appropriate browser/mailer
-    if user_runs_gnome():
-        gajim.config.set('openwith', 'gnome-open')
-    elif user_runs_kde():
-        gajim.config.set('openwith', 'kfmclient exec')
-    elif user_runs_xfce():
-        gajim.config.set('openwith', 'exo-open')
-    else:
-        gajim.config.set('openwith', 'custom')
-
-def user_runs_gnome():
-    return 'gnome-session' in get_running_processes()
-
-def user_runs_kde():
-    return 'startkde' in get_running_processes()
-
-def user_runs_xfce():
-    procs = get_running_processes()
-    if 'startxfce4' in procs or 'xfce4-session' in procs:
-        return True
-    return False
-
 def get_running_processes():
     """
     Return running processes or None (if /proc does not exist)
@@ -768,7 +744,7 @@ Description=xmpp
             gajim.config.set('check_if_gajim_is_default', False)
 
     try:
-        import gconf
+        __import__('gconf')
         # in try because daemon may not be there
         client = gconf.client_get_default()
     except Exception:
@@ -878,8 +854,8 @@ def on_avatar_save_as_menuitem_activate(widget, jid, default_name=''):
                 return
             dialog2 = dialogs.FTOverwriteConfirmationDialog(
                 _('This file already exists'), _('What do you want to do?'),
-                propose_resume=False, on_response=(on_continue, file_path))
-            dialog2.set_transient_for(dialog)
+                propose_resume=False, on_response=(on_continue, file_path),
+                transient_for=dialog)
             dialog2.set_destroy_with_parent(True)
         else:
             dirname = os.path.dirname(file_path)
@@ -1067,9 +1043,6 @@ def make_jabber_state_images():
         iconset = gajim.config.DEFAULT_ICONSET
         gajim.config.set('iconset', iconset)
 
-    path = os.path.join(helpers.get_iconset_path(iconset), '32x32')
-    gajim.interface.jabber_state_images['32'] = load_iconset(path)
-
     path = os.path.join(helpers.get_iconset_path(iconset), '16x16')
     gajim.interface.jabber_state_images['16'] = load_iconset(path)
 
@@ -1077,6 +1050,33 @@ def make_jabber_state_images():
     gajim.interface.jabber_state_images['opened'] = load_iconset(path, pixo)
     gajim.interface.jabber_state_images['closed'] = load_iconset(path, pixc)
 
+    path = os.path.join(helpers.get_iconset_path(iconset), '32x32')
+    gajim.interface.jabber_state_images['32'] = load_iconset(path)
+
+    path = os.path.join(helpers.get_iconset_path(iconset), '24x24')
+    if (os.path.exists(path)):
+        gajim.interface.jabber_state_images['24'] = load_iconset(path)
+    else:
+        # Resize 32x32 icons to 24x24
+        for each in gajim.interface.jabber_state_images['32']:
+            img = gtk.Image()
+            pix = gajim.interface.jabber_state_images['32'][each]
+            pix_type = pix.get_storage_type()
+            if pix_type == gtk.IMAGE_ANIMATION:
+                animation = pix.get_animation()
+                pixbuf = animation.get_static_image()
+            elif pix_type == gtk.IMAGE_EMPTY:
+                pix = gajim.interface.jabber_state_images['16'][each]
+                pix_16_type = pix.get_storage_type()
+                if pix_16_type == gtk.IMAGE_ANIMATION:
+                    animation = pix.get_animation()
+                    pixbuf = animation.get_static_image()
+            else:
+                pixbuf = pix.get_pixbuf()
+            scaled_pix = pixbuf.scale_simple(24, 24, gtk.gdk.INTERP_BILINEAR)
+            img.set_from_pixbuf(scaled_pix)
+            gajim.interface.jabber_state_images['24'][each] = img
+
 def reload_jabber_state_images():
     make_jabber_state_images()
     gajim.interface.roster.update_jabber_state_images()
@@ -1106,6 +1106,7 @@ def __label_size_allocate(widget, allocation):
         return
 
     # set wrap width to the pango.Layout of the labels ###
+    widget.set_alignment(0.0, 0.0)
     layout.set_width (allocation.width * pango.SCALE)
     lh = layout.get_size()[1]
 
