@@ -1,7 +1,7 @@
 # -*- coding:utf-8 -*-
 ## src/message_window.py
 ##
-## Copyright (C) 2003-2012 Yann Leboulanger <asterix AT lagaule.org>
+## Copyright (C) 2003-2014 Yann Leboulanger <asterix AT lagaule.org>
 ## Copyright (C) 2005-2008 Travis Shirk <travis AT pobox.com>
 ##                         Nikos Kouremenos <kourem AT gmail.com>
 ## Copyright (C) 2006 Geobert Quach <geobert AT gmail.com>
@@ -84,6 +84,8 @@ class MessageWindow(object):
             self.parent_paned = parent_paned
             self.notebook.reparent(self.parent_paned)
             self.parent_paned.pack2(self.notebook, resize=True, shrink=True)
+            gajim.interface.roster.xml.get_object('show_roster_menuitem').\
+                set_sensitive(True)
             orig_window.destroy()
             del orig_window
 
@@ -110,7 +112,7 @@ class MessageWindow(object):
         for key in keys:
             keyval, mod = gtk.accelerator_parse(key)
             accel_group.connect_group(keyval, mod, gtk.ACCEL_VISIBLE,
-                    self.accel_group_func)
+                self.accel_group_func)
         self.window.add_accel_group(accel_group)
 
         # gtk+ doesn't make use of the motion notify on gtkwindow by default
@@ -118,10 +120,10 @@ class MessageWindow(object):
         self.window.add_events(gtk.gdk.POINTER_MOTION_MASK)
 
         id_ = self.notebook.connect('switch-page',
-                self._on_notebook_switch_page)
+            self._on_notebook_switch_page)
         self.handlers[id_] = self.notebook
         id_ = self.notebook.connect('key-press-event',
-                self._on_notebook_key_press)
+            self._on_notebook_key_press)
         self.handlers[id_] = self.notebook
 
         # Tab customizations
@@ -144,8 +146,6 @@ class MessageWindow(object):
         self.notebook.set_show_border(gajim.config.get('tabs_border'))
         self.show_icon()
 
-        gobject.idle_add(self.notebook.grab_focus)
-
     def change_account_name(self, old_name, new_name):
         if old_name in self._controls:
             self._controls[new_name] = self._controls[old_name]
@@ -216,12 +216,16 @@ class MessageWindow(object):
                 win.destroy()
 
             if not gajim.config.get('confirm_close_multiple_tabs'):
+                for ctrl in self.controls():
+                    if ctrl.minimizable():
+                        ctrl.minimize()
                 # destroy window
                 return False
             dialogs.YesNoDialog(
                 _('You are going to close several tabs'),
                 _('Do you really want to close them all?'),
-                checktext=_('_Do not ask me again'), on_response_yes=on_yes1)
+                checktext=_('_Do not ask me again'), on_response_yes=on_yes1,
+                transient_for=self.window)
             return True
 
         def on_yes(ctrl):
@@ -307,6 +311,7 @@ class MessageWindow(object):
         # NOTE: we do not call set_control_active(True) since we don't know
         # whether the tab is the active one.
         self.show_title()
+        gobject.timeout_add(500, control.msg_textview.grab_focus)
 
     def on_tab_eventbox_button_press_event(self, widget, event, child):
         if event.button == 3: # right click
@@ -351,7 +356,10 @@ class MessageWindow(object):
         # CTRL mask
         if modifier & gtk.gdk.CONTROL_MASK:
             if keyval == gtk.keysyms.h: # CTRL + h
-                control._on_history_menuitem_activate()
+                if gtk.settings_get_default().get_property(
+                'gtk-key-theme-name') != 'Emacs':
+                    control._on_history_menuitem_activate()
+                    return True
             elif control.type_id == message_control.TYPE_CHAT and \
             keyval == gtk.keysyms.f: # CTRL + f
                 # CTRL + f moves cursor one char forward when user uses Emacs
@@ -359,28 +367,36 @@ class MessageWindow(object):
                 if not gtk.settings_get_default().get_property(
                 'gtk-key-theme-name') == 'Emacs':
                     control._on_send_file_menuitem_activate(None)
+                    return True
             elif control.type_id == message_control.TYPE_CHAT and \
             keyval == gtk.keysyms.g: # CTRL + g
                 control._on_convert_to_gc_menuitem_activate(None)
+                return True
             elif control.type_id in (message_control.TYPE_CHAT,
             message_control.TYPE_PM) and keyval == gtk.keysyms.i: # CTRL + i
                 control._on_contact_information_menuitem_activate(None)
+                return True
             elif keyval == gtk.keysyms.l or keyval == gtk.keysyms.L: # CTRL + l|L
                 control.conv_textview.clear()
+                return True
             elif keyval == gtk.keysyms.u: # CTRL + u: emacs style clear line
                 control.clear(control.msg_textview)
+                return True
             elif control.type_id == message_control.TYPE_GC and \
             keyval == gtk.keysyms.b: # CTRL + b
                 control._on_bookmark_room_menuitem_activate(None)
+                return True
             # Tab switch bindings
             elif keyval == gtk.keysyms.F4: # CTRL + F4
                 self.remove_tab(control, self.CLOSE_CTRL_KEY)
+                return True
             elif keyval == gtk.keysyms.w: # CTRL + w
                 # CTRL + w removes latest word before sursor when User uses emacs
                 # theme
                 if not gtk.settings_get_default().get_property(
                 'gtk-key-theme-name') == 'Emacs':
                     self.remove_tab(control, self.CLOSE_CTRL_KEY)
+                    return True
             elif keyval in (gtk.keysyms.Page_Up, gtk.keysyms.Page_Down):
                 # CTRL + PageUp | PageDown
                 # Create event and send it to notebook
@@ -390,12 +406,14 @@ class MessageWindow(object):
                 event.state = gtk.gdk.CONTROL_MASK
                 event.keyval = int(keyval)
                 self.notebook.emit('key_press_event', event)
+                return True
 
             if modifier & gtk.gdk.SHIFT_MASK:
                 # CTRL + SHIFT
                 if control.type_id == message_control.TYPE_GC and \
                 keyval == gtk.keysyms.n: # CTRL + SHIFT + n
                     control._on_change_nick_menuitem_activate(None)
+                    return True
         # MOD1 (ALT) mask
         elif modifier & gtk.gdk.MOD1_MASK:
             # Tab switch bindings
@@ -404,26 +422,36 @@ class MessageWindow(object):
                 if new >= self.notebook.get_n_pages():
                     new = 0
                 self.notebook.set_current_page(new)
+                return True
             elif keyval == gtk.keysyms.Left: # ALT + LEFT
                 new = self.notebook.get_current_page() - 1
                 if new < 0:
                     new = self.notebook.get_n_pages() - 1
                 self.notebook.set_current_page(new)
+                return True
             elif chr(keyval) in st: # ALT + 1,2,3..
                 self.notebook.set_current_page(st.index(chr(keyval)))
+                return True
             elif keyval == gtk.keysyms.c: # ALT + C toggles chat buttons
                 control.chat_buttons_set_visible(not control.hide_chat_buttons)
+                return True
             elif keyval == gtk.keysyms.m: # ALT + M show emoticons menu
                 control.show_emoticons_menu()
+                return True
             elif keyval == gtk.keysyms.d: # ALT + D show actions menu
-                control.on_actions_button_clicked(control.actions_button)
+                if gtk.settings_get_default().get_property(
+                'gtk-key-theme-name') != 'Emacs':
+                    control.on_actions_button_clicked(control.actions_button)
+                    return True
             elif control.type_id == message_control.TYPE_GC and \
             keyval == gtk.keysyms.t: # ALT + t
                 control._on_change_subject_menuitem_activate(None)
+                return True
         # Close tab bindings
         elif keyval == gtk.keysyms.Escape and \
-                        gajim.config.get('escape_key_closes'): # Escape
+        gajim.config.get('escape_key_closes'): # Escape
             self.remove_tab(control, self.CLOSE_ESC)
+            return True
 
     def _on_close_button_clicked(self, button, control):
         """
@@ -590,6 +618,8 @@ class MessageWindow(object):
                 # Don't close parent window, just remove the child
                 child = self.parent_paned.get_child2()
                 self.parent_paned.remove(child)
+                gajim.interface.roster.xml.get_object('show_roster_menuitem').\
+                    set_sensitive(False)
             else:
                 self.window.destroy()
             return # don't show_title, we are dead
@@ -977,7 +1007,7 @@ class MessageWindowMgr(gobject.GObject):
             return
         win.resize(size[0], size[1])
         if win.parent_paned:
-            win.parent_paned.set_position(parent_size[0])
+            win.parent_paned.set_position(gajim.config.get('roster_hpaned_position'))
 
     def _position_window(self, win, acct, type_):
         """
@@ -1231,6 +1261,8 @@ class MessageWindowMgr(gobject.GObject):
                 # Don't close parent window, just remove the child
                 child = w.parent_paned.get_child2()
                 w.parent_paned.remove(child)
+                gajim.interface.roster.xml.get_object('show_roster_menuitem').\
+                    set_sensitive(False)
                 gtkgui_helpers.resize_window(w.window,
                         gajim.config.get('roster_width'),
                         gajim.config.get('roster_height'))
@@ -1244,3 +1276,17 @@ class MessageWindowMgr(gobject.GObject):
                                         ctrl.type_id)
             ctrl.parent_win = mw
             mw.new_tab(ctrl)
+
+    def save_opened_controls(self):
+        if not gajim.config.get('remember_opened_chat_controls'):
+            return
+        chat_controls = {}
+        for acct in gajim.connections:
+            chat_controls[acct] = []
+        for ctrl in self.get_controls(type_=message_control.TYPE_CHAT):
+            acct = ctrl.account
+            if ctrl.contact.jid not in chat_controls[acct]:
+                chat_controls[acct].append(ctrl.contact.jid)
+        for acct in gajim.connections:
+            gajim.config.set_per('accounts', acct, 'opened_chat_controls',
+                ','.join(chat_controls[acct]))
