@@ -4,7 +4,7 @@
 ## Copyright (C) 2006 Jeffrey C. Ollie <jeff AT ocjtech.us>
 ##                    Nikos Kouremenos <kourem AT gmail.com>
 ##                    Stefan Bethge <stefan AT lanpartei.de>
-## Copyright (C) 2006-2012 Yann Leboulanger <asterix AT lagaule.org>
+## Copyright (C) 2006-2014 Yann Leboulanger <asterix AT lagaule.org>
 ##
 ## This file is part of Gajim.
 ##
@@ -45,9 +45,10 @@ def device_no_longer_active(self, *args):
 
 def state_changed(state):
     """
-    For Network Manager 0.7
+    For Network Manager 0.7 - 0.9
     """
-    if props.Get("org.freedesktop.NetworkManager", "State") == 3:
+    nm_state = props.Get("org.freedesktop.NetworkManager", "State")
+    if nm_state == 3 or nm_state == 70:
         for connection in gajim.connections.itervalues():
             if gajim.config.get_per('accounts', connection.name,
             'listen_to_network_manager') and connection.time_to_reconnect:
